<?php declare(strict_types = 1);

namespace Drupal\c2pa_sign\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Jrglasgow\C2paTool\Tool;

/**
 * Configure C2PA Sign settings for this site.
 */
final class SettingsForm extends ConfigFormBase {


  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'c2pa_sign_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['c2pa_sign.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('c2pa_sign.settings');

    $form['certificate_file_directory'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Certificate file directory'),
      '#description' => $this->t('The directory in which you keep your certificates and keys. When looking to sign media this directory will be searched for valid certificate key pairs. A key pair will consist of 2 files with the sam name but different extensions, i.e. cert-2024.pem and cert-2024.key. Only key pairs that are complete will be used. If there are multiple pairs the certificate that expires last will be used. All Certificates MUST fulfill all <a href="https://opensource.contentauthenticity.org/docs/manifest/signing-manifests/#certificates" target="_blank">requirements in the documentation</a>.'),
      '#default_value' => $config->get('certificate_file_directory'),
    ];
    $certs = c2pa_sign_get_certs($config->get('certificate_file_directory'));
    if (!empty($certs['cert_to_use']) && is_object($certs['cert_to_use'])) {
      $form['cert_info'] = c2pa_sign_certificate_info($certs['cert_to_use']);
    }
    $logger = $this->logger('c2pa_sign');
    $tool = new Tool($logger);
    if (!empty($config->get('c2patool_binary_location')) && file_exists($config->get('c2patool_binary_location')) && is_executable($config->get('c2patool_binary_location'))) {
      $tool->setBinary($config->get('c2patool_binary_location'));
    }
    $form['c2patool_binary_location'] = [
      '#type' => 'textfield',
      '#title' => $this->t('c2patool binary location'),
      '#description' => $this->t('The location on the system of the <a href="https://github.com/contentauth/c2patool" target="_blank">c2patool binary which can be found on Github</a>.'),
      '#default_value' => $tool->getBinary(),
    ];

    if ($tool->getBinary()) {
      $form['c2patool_info'] = [
        '#markup' => t('c2patool version: %version.', ['%version' => $tool->getBinaryVersion()]),
      ];
    }

    // TODO run the function (to be created) to grab the ideal cert/key pair
    // (certificate expires last) and display cert information
    // i.e. file names, org names, valid dates, etc...

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $certs = c2pa_sign_get_certs($form_state->getValue('certificate_file_directory'));
    if (empty($certs['cert_to_use'])) {
      $form_state->setErrorByName('certificate_file_directory', $this->t('No valid cert/key pairs in the directory %directory.', ['%directory' => $form_state->getValue('certificate_file_directory')]));

    }
    if (!empty($certs['errors'])) {
      foreach ($certs['errors'] AS $error) {
        if (isset($error['text']) && !empty($error['text'])) {
          $form_state->setErrorByName('certificate_file_directory', $error['text'], $error['args'] ?? []);
          $this->messenger()->addWarning($error['text'], $error['args'] ?? []);
        }
      }
    }

    // validate c2patool binary
    $binary_location = $form_state->getValue('c2patool_binary_location');
    if (empty($binary_location)) {
      $this->messenger()->addWarning('c2patool binary location is empty.');
      $this->searchBinary($form_state);
      $this->messenger()->addWarning('c2patool binary location is empty.');
      if (empty($form_state->getValue('c2patool_binary_location'))) {
        $form_state->setErrorByName('c2patool_binary_location', $this->t('c2patool binary location is required.'));
      }
    }
    else if (!file_exists($binary_location)) {
      $this->messenger()->addWarning($this->t('c2patool binary location (%location) does not exist.', ['%location' => $binary_location]));
      $this->searchBinary($form_state);
      if ($binary_location == $form_state->getValue('c2patool_binary_location')) {
        // the binary location didn't change
        $form_state->setErrorByName('c2patool_binary_location', $this->t('c2patool binary location is required and MUST exist.'));
      }
    }
    else if (!is_executable($binary_location)) {
      $this->messenger()->addWarning($this->t('c2patool binary location (%location) is not executable.', ['%location' => $binary_location]));
      $this->searchBinary($form_state);
      if ($binary_location == $form_state->getValue('c2patool_binary_location')) {
        // the binary location didn't change
        $form_state->setErrorByName('c2patool_binary_location', $this->t('c2patool binary location is required and MUST be executable.'));
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * Search for the c2patool binary ans set it if found
   *
   * @param $form_state
   *
   * @return void
   */
  protected function searchBinary($form_state) {
    $tool = new Tool($this->logger('c2pa_sign'));
    $binary_location = $tool->getBinary();
    if (!empty($binary_location)) {
      $form_state->setValue('c2patool_binary_location', $binary_location);
      \Drupal::messenger()->addStatus('c2patool binary found at %location', ['%location' => $binary_location]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('c2pa_sign.settings')
      ->set('c2patool_binary_location', $form_state->getValue('c2patool_binary_location'))
      ->set('certificate_file_directory', $form_state->getValue('certificate_file_directory'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
